@@ -44,7 +44,6 @@
 #include "rte_mbuf.h"
 #include "dpdk_eal.h"
 #include "rte_ethdev.h"
-#include "stats_dump.h"
 #include "cpuinfo.h"
 #include <stdbool.h>
 #include <pthread.h>
@@ -139,6 +138,7 @@ struct relay_virtio {
 	struct rte_mbuf *tx_pkts[BURST_LEN];
 	unsigned tx_pkts_avail, tx_pkts_used;
 	rte_spinlock_t sl;
+	volatile bool lm_pending;
 } __attribute__ ((aligned (RTE_CACHE_LINE_SIZE)));
 
 struct relay_dpdk {
@@ -354,29 +354,48 @@ static void get_tx_conf(const struct rte_eth_dev_info *dev_info,
 	uint64_t desired_offloads;
 	desired_offloads = DEV_TX_OFFLOAD_IPV4_CKSUM |
 				DEV_TX_OFFLOAD_UDP_CKSUM |
-				DEV_TX_OFFLOAD_TCP_CKSUM |
-				DEV_TX_OFFLOAD_TCP_TSO;
+				DEV_TX_OFFLOAD_TCP_CKSUM;
+	if (g_vio_worker_conf.enable_tso)
+		desired_offloads |= (DEV_TX_OFFLOAD_TCP_TSO |
+					DEV_TX_OFFLOAD_UDP_TSO);
 	tx_conf->offloads = dev_info->tx_offload_capa & desired_offloads;
 #endif
+#if RTE_VERSION < RTE_VERSION_NUM(18,8,0,0)
 	tx_conf->txq_flags &= ~ETH_TXQ_FLAGS_NOOFFLOADS;
+#endif
 }
 
 static void get_rx_conf(const struct rte_eth_dev_info *dev_info,
 			struct rte_eth_rxconf *rx_conf)
 {
 	*rx_conf = dev_info->default_rxconf;
+#if RTE_VERSION >= RTE_VERSION_NUM(17,11,0,0)
+	uint64_t desired_offloads;
+	desired_offloads = DEV_RX_OFFLOAD_IPV4_CKSUM |
+				DEV_RX_OFFLOAD_UDP_CKSUM |
+				DEV_RX_OFFLOAD_TCP_CKSUM;
+	rx_conf->offloads = dev_info->rx_offload_capa & desired_offloads;
+#endif
 }
 
 static int cleanup_eth_dev(dpdk_port_t port_id)
 {
 	int err;
-	char detach_dbdf[RTE_ETH_NAME_MAX_LEN];
 
 	rte_eth_dev_stop(port_id);
 	rte_eth_dev_close(port_id);
+#if RTE_VERSION >= RTE_VERSION_NUM(18,11,0,0)
+	const char rte_detach_api[] = "rte_dev_remove";
+	struct rte_eth_dev_info dev_info;
+	rte_eth_dev_info_get(port_id, &dev_info);
+	err = rte_dev_remove(dev_info.device);
+#else
+	const char rte_detach_api[] = "rte_eth_dev_detach";
+	char detach_dbdf[RTE_ETH_NAME_MAX_LEN];
 	err = rte_eth_dev_detach(port_id, detach_dbdf);
+#endif
 	if (err != 0) {
-		log_warning("rte_eth_dev_detach(%hhu) failed with error %i",
+		log_warning("%s(%hhu) failed with error %i", rte_detach_api,
 			port_id, err);
 	}
 
@@ -432,6 +451,7 @@ migrate_mempool(vio_vf_relay_t *relay, int newnode, unsigned num_pktmbufs)
 	 * See http://dpdk.org/doc/api/rte__ethdev_8h.html
 	 */
 	assert(relay->dpdk.state != DPDK_READY);
+	assert(relay->dpdk.rx_pkts_avail == 0);
 	if (relay->dpdk.state == DPDK_ADDED) {
 		log_debug("Previously setup VF requires update. Stopping VF...");
 		rte_eth_dev_stop(port_id);
@@ -505,7 +525,29 @@ static int dev_queue_configure(const char *name, dpdk_port_t port_id,
 	log_info("Adding DPDK port %hhu ('%s') to virtio ('%u')",
 		port_id, name, virtio_id);
 
+	/* Configure per-port offloads. */
+#if RTE_VERSION >= RTE_VERSION_NUM(18,8,0,0)
+	rte_eth_dev_info_get(port_id, &dev_info);
+	if (dev_info.rx_offload_capa & DEV_RX_OFFLOAD_JUMBO_FRAME)
+		eth_conf.rxmode.offloads |= DEV_RX_OFFLOAD_JUMBO_FRAME;
+	if (dev_info.rx_offload_capa & DEV_RX_OFFLOAD_CHECKSUM)
+		eth_conf.rxmode.offloads |= DEV_RX_OFFLOAD_CHECKSUM;
+	if (dev_info.tx_offload_capa & DEV_TX_OFFLOAD_IPV4_CKSUM)
+		eth_conf.txmode.offloads |= DEV_TX_OFFLOAD_IPV4_CKSUM;
+	if (dev_info.tx_offload_capa & DEV_TX_OFFLOAD_UDP_CKSUM)
+		eth_conf.txmode.offloads |= DEV_TX_OFFLOAD_UDP_CKSUM;
+	if (dev_info.tx_offload_capa & DEV_TX_OFFLOAD_TCP_CKSUM)
+		eth_conf.txmode.offloads |= DEV_TX_OFFLOAD_TCP_CKSUM;
+	if (g_vio_worker_conf.enable_tso) {
+		if (dev_info.tx_offload_capa & DEV_TX_OFFLOAD_TCP_TSO)
+			eth_conf.txmode.offloads |= DEV_TX_OFFLOAD_TCP_TSO;
+		if (dev_info.tx_offload_capa & DEV_TX_OFFLOAD_UDP_TSO)
+			eth_conf.txmode.offloads |= DEV_TX_OFFLOAD_UDP_TSO;
+	}
+#else
 	eth_conf.rxmode.jumbo_frame = 1;
+	eth_conf.rxmode.hw_ip_checksum = 1;
+#endif
 	eth_conf.rxmode.max_rx_pkt_len = relay->use_jumbo ? JUMBO_MBUF_SIZE :
 						DEFAULT_MBUF_SIZE;
 	err = rte_eth_dev_configure(port_id, 1, 1, &eth_conf);
@@ -571,23 +613,42 @@ static int init_vf(const char *pci_dbdf, dpdk_port_t *port_id,
 {
 	int err;
 
+#if RTE_VERSION >= RTE_VERSION_NUM(18,8,0,0)
+	const char rte_attach_api[] = "rte_dev_probe";
+	err = rte_dev_probe(pci_dbdf);
+#else
+	const char rte_attach_api[] = "rte_eth_dev_attach";
 	err = rte_eth_dev_attach(pci_dbdf, port_id);
+#endif
 	if (err != 0) {
 		/* err is always -1 on error, so no useful additional info.
 		 * Print it anyway for consistency with other error messages. */
-		log_error("rte_eth_dev_attach('%s') failed with error %i",
+		log_error("%s('%s') failed with error %i", rte_attach_api,
 			pci_dbdf, err);
 		check_uio_driver_setup(pci_dbdf);
 		return 2;
 	}
 
+#if RTE_VERSION >= RTE_VERSION_NUM(18,8,0,0)
+	struct rte_dev_iterator it;
+	RTE_ETH_FOREACH_MATCHING_DEV(*port_id, pci_dbdf, &it) {
+#endif
 	err = dev_queue_configure(pci_dbdf, *port_id, virtio_id, relay, false);
 	if (err) {
+#if RTE_VERSION >= RTE_VERSION_NUM(18,8,0,0)
+		rte_eth_iterator_cleanup(&it);
+#endif
 		cleanup_eth_dev(*port_id);
 		return err;
 	}
 
 	rte_eth_promiscuous_enable(*port_id);
+#if RTE_VERSION >= RTE_VERSION_NUM(18,8,0,0)
+	/* Enforce a single match - there should only be one. */
+	rte_eth_iterator_cleanup(&it);
+	break;
+	}
+#endif
 
 	return 0;
 }
@@ -612,6 +673,7 @@ static int configure_dev_with_virtio(const char *pci_dbdf, dpdk_port_t port_id,
 
 	/* Success */
 	log_info("Added dpdk port %hhu to relay", port_id);
+	relay->vio.lm_pending = false;
 	relay->dpdk.dpdk_port = port_id;
 	strncpy(relay->dpdk.pci_dbdf, pci_dbdf, 20);
 	find_vf2virtio_cpu(relay);
@@ -1011,18 +1073,32 @@ static inline uint32_t calc_eth_header_hash(struct ether_hdr *eth_hdr)
 	return rte_jhash_32b(buf, hashwords, 0xdeadbee5);
 }
 
-static inline void calc_mbuf_hash(struct rte_mbuf **pkts, uint16_t nb_pkts)
+static inline void
+calc_mbuf_queue(vio_vf_relay_t *relay, struct rte_mbuf **pkts, uint16_t nb_pkts)
 {
 	uint16_t i;
+	uint32_t h;
+	struct ether_hdr *eth_hdr;
 
 	for (i=0; i<nb_pkts; ++i) {
 		rte_prefetch0(rte_pktmbuf_mtod(pkts[i], void *));
 	}
-	for (i=0; i<nb_pkts; ++i) {
-		struct ether_hdr *eth_hdr;
 
+	for (i=0; i<nb_pkts; ++i) {
 		eth_hdr = rte_pktmbuf_mtod(pkts[i], struct ether_hdr *);
-		pkts[i]->hash.usr = calc_eth_header_hash(eth_hdr);
+		h = calc_eth_header_hash(eth_hdr);
+
+		/* Determine queue that is to be used for each packet. */
+		if (likely(relay->vio.pow2queues))
+			h = h & (relay->vio.rx_q_active - 1); /* Retain lower bits of h: cheap modulo. */
+		else
+			h = h % relay->vio.rx_q_active;
+		/* Here, h is a number < rx_q_active, so it can be used
+		 * to index the lookup table. */
+		h = relay->vio.rx_q_lut[h];
+
+		/* Batch same queue packets. */
+		pkts[i]->hash.fdir.id = h;
 	}
 }
 
@@ -1278,6 +1354,7 @@ static inline int relay_vm2vf_traffic(vio_vf_relay_t *relay)
 			--avail;
 			rte_pktmbuf_free(pkts[avail]);
 		}
+		relay->vio.tx_pkts_avail = 0;
 	}
 
 	if (unlikely(relay->vio.state == VIRTIO_REMOVING1)) {
@@ -1303,15 +1380,22 @@ static inline int dpdk_rx(vio_vf_relay_t *relay)
 		return -1;
 #endif
 
+	/* The following check prevents a segmentation fault during live
+	 * migration when using DPDK >= 17.11. */
+	if (likely(!relay->vio.lm_pending)) {
 #if RTE_VERSION >= RTE_VERSION_NUM(16,7,0,0)
-	try_rcv = rte_vhost_avail_entries((int)relay->vio.vio_dev,
-						VIRTIO_RXQ);
+		try_rcv = rte_vhost_avail_entries((int)relay->vio.vio_dev,
+							VIRTIO_RXQ);
 #else
-	try_rcv = vring_available_entries((struct virtio_net *)relay->vio.vio_dev,
-						VIRTIO_RXQ);
+		try_rcv = vring_available_entries(
+					(struct virtio_net *)relay->vio.vio_dev,
+					VIRTIO_RXQ);
 #endif
-	if (try_rcv > BURST_LEN)
+		if (try_rcv > BURST_LEN)
+			try_rcv = BURST_LEN;
+	} else {
 		try_rcv = BURST_LEN;
+	}
 #ifndef VIRTIO_ECHO
 	rcvd = rte_eth_rx_burst(relay->dpdk.dpdk_port, 0, pkts, try_rcv);
 #else
@@ -1322,7 +1406,7 @@ static inline int dpdk_rx(vio_vf_relay_t *relay)
 	relay->dpdk.rx_pkts_used = 0;
 
 	if (relay->vio.rx_q_bitmap > 1)
-		calc_mbuf_hash(pkts, rcvd); /* For multiqueue. */
+		calc_mbuf_queue(relay, pkts, rcvd);
 
 	/* Update stats. */
 	if (rcvd) {
@@ -1346,55 +1430,63 @@ static inline int virtio_tx(vio_vf_relay_t *relay)
 		return -1;
 
 	if (multiqueue) {
-		unsigned i;
-		unsigned q[relay->dpdk.rx_pkts_avail];
-		unsigned _sent;
+		unsigned i, k=0, q;
+		int j;
+		uint8_t q_len[relay->vio.rx_q_active];
+		struct rte_mbuf *q_pkts[relay->vio.rx_q_active][BURST_LEN];
 
-		/* Find the queue that is to be used for each available packet. */
+		/* Batch same queue packets.
+		 * Optimization hint: This may be done redundantly when we fail
+		 * to enqueue all the packets in one go. Per-relay queues are
+		 * required to implement that though, and it won't play nice
+		 * with our disconnect logic... */
+		memset(q_len, 0, relay->vio.rx_q_active * sizeof(uint8_t));
 		for (i=0; i<relay->dpdk.rx_pkts_avail; ++i) {
-			unsigned h = pkts[i]->hash.usr;
-			if (likely(relay->vio.pow2queues))
-				h = h & (relay->vio.rx_q_active - 1); /* Retain lower bits of h. Similar to modulo -> result falls within num queues=rx_q_active. */
-			else
-				h = h % relay->vio.rx_q_active;
-			/* Here, h is a number < rx_q_active, so it can be used
-			 * to index the lookup table. */
-			h = relay->vio.rx_q_lut[h];
-			q[i] = h;
+			q = pkts[i]->hash.fdir.id;
+			q_pkts[q][q_len[q]++] = pkts[i];
 		}
 
-		/* Search run-lengths of same q. */
-		unsigned cur_q = q[0];
-		unsigned runlen = 1;
-		for (i=1; i<relay->dpdk.rx_pkts_avail; ++i) {
-			if (q[i] == cur_q) {
-				++runlen;
-			} else {
+		/* Send for each queue. */
+		for (i=0; i<relay->vio.rx_q_active; ++i) {
+			if (!q_len[i])
+				continue;
+
 #if defined(VIRTIO_RETRY_ENQUEUE)
-				_sent = worker_vhost_enqueue_burst(
-						relay->vio.vio_dev,
-						cur_q*2, pkts+i-runlen, runlen);
+			sent = worker_vhost_enqueue_burst(relay->vio.vio_dev,
+					i*2, q_pkts[i], q_len[i]);
 #else
-				_sent = rte_vhost_enqueue_burst(
-						relay->vio.vio_dev,
-						cur_q*2, pkts+i-runlen, runlen);
+			sent = rte_vhost_enqueue_burst(relay->vio.vio_dev,
+					i*2, q_pkts[i], q_len[i]);
 #endif
-				sent += _sent;
-				cur_q = q[i];
-				runlen = 1;
-				if (_sent != runlen) { /* Stop immediately when an error is encountered. Remaining packets will be processed in a subsequent run. */
-				        runlen = 0;
-				        break;
-				}
+
+			/* Update sent to VM stats. */
+			if (sent) {
+				unsigned bytes=0;
+				for (j=0; j<sent; ++j)
+					bytes += q_pkts[i][j]->pkt_len;
+				relay->stats.vio_tx_bytes += bytes;
+				relay->stats.vio_tx += sent;
+			}
+
+			/* Add unsuccessful packets back to the internal buffer.
+			 * Note that this breaks strict ordering within the
+			 * buffer. Order within a flow remains. */
+			j = q_len[i];
+			while (j > sent) {
+				j--;
+				pkts[k++] = q_pkts[i][j];
+			}
+
+			/* Free packets that have been enqueued. */
+			while (sent) {
+				--sent;
+				rte_pktmbuf_free(q_pkts[i][sent]);
 			}
 		}
-#if defined(VIRTIO_RETRY_ENQUEUE)
-		sent += worker_vhost_enqueue_burst(relay->vio.vio_dev,
-						cur_q*2, pkts+i-runlen, runlen);
-#else
-		sent += rte_vhost_enqueue_burst(relay->vio.vio_dev,
-						cur_q*2, pkts+i-runlen, runlen);
-#endif
+
+		relay->dpdk.rx_pkts_avail = k;
+		/* The used counter remains the same, since we add failed
+		 * packets to the former 'used' position. */
 	} else {
 #if defined(VIRTIO_RETRY_ENQUEUE)
 		sent += worker_vhost_enqueue_burst(relay->vio.vio_dev,
@@ -1405,24 +1497,24 @@ static inline int virtio_tx(vio_vf_relay_t *relay)
 						VIRTIO_RXQ, pkts,
 						relay->dpdk.rx_pkts_avail);
 #endif
-	}
-	relay->dpdk.rx_pkts_avail -= sent;
-	relay->dpdk.rx_pkts_used += sent;
-	assert(relay->dpdk.rx_pkts_used <= BURST_LEN);
+		relay->dpdk.rx_pkts_avail -= sent;
+		relay->dpdk.rx_pkts_used += sent;
+		assert(relay->dpdk.rx_pkts_used <= BURST_LEN);
 
-	/* Update sent to VM stats. */
-	if (sent) {
-		unsigned bytes=0;
-		for (int i=0; i<sent; ++i)
-			bytes += pkts[i]->pkt_len;
-		relay->stats.vio_tx_bytes += bytes;
-		relay->stats.vio_tx += sent;
-	}
+		/* Update sent to VM stats. */
+		if (sent) {
+			unsigned bytes=0;
+			for (int i=0; i<sent; ++i)
+				bytes += pkts[i]->pkt_len;
+			relay->stats.vio_tx_bytes += bytes;
+			relay->stats.vio_tx += sent;
+		}
 
-	/* Free packets that have been enqueued. */
-	while (sent) {
-		--sent;
-		rte_pktmbuf_free(pkts[sent]);
+		/* Free packets that have been enqueued. */
+		while (sent) {
+			--sent;
+			rte_pktmbuf_free(pkts[sent]);
+		}
 	}
 
 	return sent;
@@ -1493,6 +1585,7 @@ static inline int relay_vf2vm_traffic(vio_vf_relay_t *relay)
 			--avail;
 			rte_pktmbuf_free(pkts[avail]);
 		}
+		relay->dpdk.rx_pkts_avail = 0;
 	}
 
 	if (unlikely(relay->vio.state == VIRTIO_REMOVING2))
@@ -1575,7 +1668,7 @@ int virtio_forwarders_initialize(void)
 	pthread_t static_vfs_init;
 	const struct virtio_vhostuser_conf *conf = &g_vio_worker_conf;
 
-	memset(virtio_vf_relays, 0, sizeof(*virtio_vf_relays));
+	memset(virtio_vf_relays, 0, sizeof(*virtio_vf_relays) * MAX_RELAYS);
 
 	/* Issue NUMA mismatch warnings. */
 	for (w=0; w<MAX_RELAYS; ++w) {
@@ -1629,12 +1722,13 @@ int virtio_forwarders_initialize(void)
 		virtio_vf_relays[w].dpdk.rx_pkts_used = 0;
 		virtio_vf_relays[w].vio.tx_pkts_avail = 0;
 		virtio_vf_relays[w].vio.tx_pkts_used = 0;
+		virtio_vf_relays[w].vio.lm_pending = false;
 		rte_spinlock_init(&virtio_vf_relays[w].vio.sl);
 		rte_spinlock_init(&virtio_vf_relays[w].dpdk.sl);
 	}
 
 	/* Launch worker_func on all slaves. */
-	memset(worker_threads, 0, sizeof(*worker_threads));
+	memset(worker_threads, 0, sizeof(*worker_threads) * MAX_CPUS);
 	worker_core_bitmap = conf->worker_core_bitmap;
 	log_debug("Master running on core %u", rte_get_master_lcore());
 	log_debug("Starting workers on CPU bitmap 0x%08llX",
@@ -1849,6 +1943,7 @@ int virtio_forwarder_add_virtio(void *virtionet, unsigned id)
 			log_warning("start_eth_dev(port %u) failed with error %i ('%s')",
 				relay->dpdk.dpdk_port, err, rte_strerror(-err));
 		} else {
+			relay->vio.lm_pending = false;
 			relay->dpdk.state = DPDK_READY;
 			__sync_synchronize();
 			worker_threads[relay->dpdk.vf2vio_cpu].need_update = true;
@@ -1991,41 +2086,6 @@ void virtio_forwarders_remove_all(void)
 	}
 }
 
-void virtio_forwarders_print_stats(int ptmfd)
-{
-	for (unsigned id=0; id<MAX_RELAYS; ++id) {
-		if (virtio_vf_relays[id].vio.state != VIRTIO_READY &&
-				virtio_vf_relays[id].dpdk.state != DPDK_READY)
-			continue;
-
-		vio_vf_relay_t *relay = &virtio_vf_relays[id];
-		print_stat(ptmfd, "relay%u.virtio_rx=%"PRIu64"\n", id,
-			relay->stats.vio_rx);
-		print_stat(ptmfd, "relay%u.virtio_rx_bytes=%"PRIu64"\n",
-			id, relay->stats.vio_rx_bytes);
-		print_stat(ptmfd, "relay%u.dpdk_tx=%"PRIu64"\n", id,
-			relay->stats.dpdk_tx);
-		print_stat(ptmfd, "relay%u.dpdk_tx_bytes=%"PRIu64"\n", id,
-			relay->stats.dpdk_tx_bytes);
-		print_stat(ptmfd, "relay%u.dpdk_drop_full=%"PRIu64"\n", id,
-			relay->stats.dpdk_drop_full);
-		print_stat(ptmfd, "relay%u.dpdk_drop_unavail=%"PRIu64"\n", id,
-			relay->stats.dpdk_drop_unavail);
-		print_stat(ptmfd, "relay%u.dpdk_rx=%"PRIu64"\n", id,
-			relay->stats.dpdk_rx);
-		print_stat(ptmfd, "relay%u.dpdk_rx_bytes=%"PRIu64"\n", id,
-			relay->stats.dpdk_rx_bytes);
-		print_stat(ptmfd, "relay%u.virtio_tx=%"PRIu64"\n", id,
-			relay->stats.vio_tx);
-		print_stat(ptmfd, "relay%u.virtio_tx_bytes=%"PRIu64"\n", id,
-			relay->stats.vio_tx_bytes);
-		print_stat(ptmfd, "relay%u.virtio_drop_full=%"PRIu64"\n", id,
-			relay->stats.vio_drop_full);
-		print_stat(ptmfd, "relay%u.virtio_drop_unavail=%"PRIu64"\n", id,
-			relay->stats.vio_drop_unavail);
-	}
-}
-
 /** Converts virtio_state_t to debug string. */
 static void
 virtio_state_to_str(vio_state_t state, char *ans, size_t cch_max_ans)
@@ -2206,7 +2266,6 @@ int virtio_get_free_relay_id(char **socket_map)
 
 bool virtio_relay_has_device(unsigned id, const char *dev)
 {
-
 	vio_vf_relay_t *relay;
 
 	if (id >= MAX_RELAYS) {
@@ -2218,3 +2277,16 @@ bool virtio_relay_has_device(unsigned id, const char *dev)
 
 	return pci_dbdf_equal(dev, relay->dpdk.pci_dbdf);
 }
+
+void virtio_set_lm_pending(int relay_id)
+{
+	vio_vf_relay_t *relay;
+
+	if (relay_id >= MAX_RELAYS) {
+		log_error("Tried to flip live migration flag for an invalid virtio ID %d!",
+			relay_id);
+		return;
+	}
+	relay = &virtio_vf_relays[relay_id];
+	relay->vio.lm_pending = true;
+}
